@@ -1,16 +1,16 @@
+import contextlib
+import re
+import time
+
 import h5py
 import numpy as np
 import pandas as pd
-import re
-import contextlib
-import time
-import warnings
 
 
-class HDF_Object(object):
-    '''
+class HDF_Object:
+    """
     A generic class to access HDF components.
-    '''
+    """
 
     @property
     def read_only(self):
@@ -29,18 +29,10 @@ class HDF_Object(object):
         return self._name
 
     def __eq__(self, other):
-        return (
-            self.file_name == other.self.file_name
-        ) and (
-            self.name == other.name
-        )
+        return (self.file_name == other.self.file_name) and (self.name == other.name)
 
     @contextlib.contextmanager
-    def editing_mode(
-        self,
-        read_only: bool = False,
-        truncate: bool = True
-    ):
+    def editing_mode(self, read_only: bool = False, truncate: bool = True):
         original_read_only = self.read_only
         original_truncate = self.truncate
         try:
@@ -53,7 +45,7 @@ class HDF_Object(object):
 
     @property
     def metadata(self):
-        with h5py.File(self.file_name, "a") as hdf_file:
+        with h5py.File(self.file_name) as hdf_file:
             return dict(hdf_file[self.name].attrs)
 
     def __init__(
@@ -82,12 +74,10 @@ class HDF_Object(object):
             raise AttributeError("Cannot set read-only attributes")
         elif not isinstance(name, str):
             raise KeyError(f"Attribute name '{name}' is not a string")
-        elif not bool(re.match(r'^[a-zA-Z_][\w.-]*$', name)):
+        elif not bool(re.match(r"^[a-zA-Z_][\w.-]*$", name)):
             raise KeyError(f"Invalid attribute name: {name}")
         if (not self.truncate) and (name in self.metadata):
-            raise KeyError(
-                f"Attribute '{name}' cannot be truncated"
-            )
+            raise KeyError(f"Attribute '{name}' cannot be truncated")
         if isinstance(value, (str, bool, int, float)):
             with h5py.File(self.file_name, "a") as hdf_file:
                 hdf_object = hdf_file[self.name]
@@ -101,7 +91,6 @@ class HDF_Object(object):
 
 
 class HDF_Group(HDF_Object):
-
     def __init__(
         self,
         *,
@@ -158,35 +147,32 @@ class HDF_Group(HDF_Object):
 
     @property
     def groups(self):
-        return [
-            self.__getattribute__(name) for name in self.group_names
-        ]
+        return [self.__getattribute__(name) for name in self.group_names]
 
     @property
     def datasets(self):
-        return [
-            self.__getattribute__(name) for name in self.dataset_names
-        ]
+        return [self.__getattribute__(name) for name in self.dataset_names]
 
     @property
     def dataframes(self):
-        return [
-            self.__getattribute__(name) for name in self.dataframe_names
-        ]
+        return [self.__getattribute__(name) for name in self.dataframe_names]
 
     @property
     def components(self):
         group_names = []
         dataset_names = []
         datafame_names = []
-        with h5py.File(self.file_name, "a") as hdf_file:
+        with h5py.File(self.file_name) as hdf_file:
             hdf_object = hdf_file[self.name]
             for name in sorted(hdf_object):
                 if isinstance(hdf_object[name], h5py.Dataset):
                     if not name.endswith("_mmap"):
                         dataset_names.append(name)
                 else:
-                    if name.endswith('_df') or "is_pd_dataframe" in hdf_object[name].attrs:
+                    if (
+                        name.endswith("_df")
+                        or "is_pd_dataframe" in hdf_object[name].attrs
+                    ):
                         datafame_names.append(name)
                     else:
                         group_names.append(name)
@@ -213,20 +199,16 @@ class HDF_Group(HDF_Object):
     def __setattr__(self, name, value):
         try:
             super().__setattr__(name, value)
-        except NotImplementedError:
+        except NotImplementedError as e:
             if not self.truncate:
                 if name in self.group_names:
-                    raise KeyError(
-                        f"Group name '{name}' cannot be truncated"
-                    )
+                    raise KeyError(f"Group name '{name}' cannot be truncated") from e
                 elif name in self.dataset_names:
-                    raise KeyError(
-                        f"Dataset name '{name}' cannot be truncated"
-                    )
+                    raise KeyError(f"Dataset name '{name}' cannot be truncated") from e
                 elif name in self.dataframe_names:
                     raise KeyError(
                         f"Dataframe name '{name}' cannot be truncated"
-                    )
+                    ) from e
             if isinstance(value, (np.ndarray, pd.core.series.Series)):
                 self.add_dataset(name, value)
             elif isinstance(value, (dict, pd.DataFrame)):
@@ -236,8 +218,8 @@ class HDF_Group(HDF_Object):
                     f"Type '{type(value)}' is invalid for attribute {name}",
                     "Only (str, bool, int, float, np.ndarray, "
                     "pd.core.series.Series, dict pd.DataFrame) types are "
-                    "accepted."
-                )
+                    "accepted.",
+                ) from e
 
     def add_dataset(
         self,
@@ -272,12 +254,12 @@ class HDF_Group(HDF_Object):
                     # chunks=array.shape,
                     maxshape=tuple([None for i in array.shape]),
                 )
-            except TypeError:
+            except TypeError as e:
                 raise NotImplementedError(
                     f"Type {array.dtype} is not understood. "
                     "If this is a string format, try to cast it to "
                     "np.dtype('O') as possible solution."
-                )
+                ) from e
             dataset = HDF_Dataset(
                 file_name=self.file_name,
                 name=f"{self.name}/{name}",
@@ -298,7 +280,7 @@ class HDF_Group(HDF_Object):
                 del hdf_object[name]
             hdf_object.create_group(name)
         if isinstance(group, pd.DataFrame):
-            if not name.endswith('_df'):
+            if not name.endswith("_df"):
                 raise TypeError(f"DataFrame group name `{name}` must end with `_df`")
             group = dict(group)
             # group["is_pd_dataframe"] = True
@@ -322,7 +304,6 @@ class HDF_Group(HDF_Object):
 
 
 class HDF_Dataset(HDF_Object):
-
     def __init__(
         self,
         *,
@@ -347,12 +328,12 @@ class HDF_Dataset(HDF_Object):
 
     @property
     def dtype(self):
-        with h5py.File(self.file_name, "a") as hdf_file:
+        with h5py.File(self.file_name) as hdf_file:
             return hdf_file[self.name].dtype
 
     @property
     def shape(self):
-        with h5py.File(self.file_name, "a") as hdf_file:
+        with h5py.File(self.file_name) as hdf_file:
             return hdf_file[self.name].shape
 
     @property
@@ -360,7 +341,7 @@ class HDF_Dataset(HDF_Object):
         return self[...]
 
     def __getitem__(self, keys):
-        with h5py.File(self.file_name, "a") as hdf_file:
+        with h5py.File(self.file_name) as hdf_file:
             hdf_object = hdf_file[self.name]
             if h5py.check_string_dtype(hdf_object.dtype) is not None:
                 hdf_object = hdf_object.asstr()
@@ -371,9 +352,7 @@ class HDF_Dataset(HDF_Object):
             raise AttributeError("Cannot append read-only dataset")
         with h5py.File(self.file_name, "a") as hdf_file:
             hdf_object = hdf_file[self.name]
-            new_shape = tuple(
-                [i + j for i, j in zip(self.shape, data.shape)]
-            )
+            new_shape = tuple([i + j for i, j in zip(self.shape, data.shape)])
             old_size = len(self)
             hdf_object.resize(new_shape)
             hdf_object[old_size:] = data
@@ -421,22 +400,15 @@ class HDF_Dataset(HDF_Object):
             offset = subgroup.id.get_offset()
             shape = subgroup.shape
             import mmap
+
             with open(self.file_name, "rb") as raw_hdf_file:
-                mmap_obj = mmap.mmap(
-                    raw_hdf_file.fileno(),
-                    0,
-                    access=mmap.ACCESS_READ
-                )
+                mmap_obj = mmap.mmap(raw_hdf_file.fileno(), 0, access=mmap.ACCESS_READ)
                 return np.frombuffer(
-                    mmap_obj,
-                    dtype=subgroup.dtype,
-                    count=np.prod(shape),
-                    offset=offset
+                    mmap_obj, dtype=subgroup.dtype, count=np.prod(shape), offset=offset
                 ).reshape(shape)
 
 
 class HDF_Dataframe(HDF_Group):
-
     @property
     def dtype(self):
         dtypes = []
@@ -490,20 +462,20 @@ class HDF_File(HDF_Group):
         """HDF file object to load/save the hdf file. It also provides convenient
         attribute-like accesses to operate the data in the HDF object.
 
-        Instead of relying directly on the `h5py` interface, we will use an HDF wrapper 
-        file to provide consistent access to only those specific HDF features we want. 
-        Since components of an HDF file come in three shapes `datasets`, `groups` and `attributes`, 
-        we will first define a generic HDF wrapper object to handle these components. 
-        Once this is done, the HDF wrapper file can be treated as such an object with additional 
+        Instead of relying directly on the `h5py` interface, we will use an HDF wrapper
+        file to provide consistent access to only those specific HDF features we want.
+        Since components of an HDF file come in three shapes `datasets`, `groups` and `attributes`,
+        we will first define a generic HDF wrapper object to handle these components.
+        Once this is done, the HDF wrapper file can be treated as such an object with additional
         features to open and close the initial connection.
 
         Args:
             file_name (str): file path.
-            read_only (bool, optional): If hdf is read-only. Defaults to True.
-            truncate (bool, optional): If existing groups and datasets can be 
-                truncated (i.e. are overwitten). Defaults to False.
-            delete_existing (bool, optional): If the file already exists, 
-                delete it completely and create a new one. Defaults to False.
+            read_only (bool, optional): If hdf is read-only. Mutually exclusive with `delete_existing` and `truncate`. Defaults to True.
+            truncate (bool, optional): If existing groups and datasets can be
+                truncated (i.e. are overwitten). Mutually exclusive with `read_only`. Defaults to False.
+            delete_existing (bool, optional): If the file already exists,
+                delete it completely and create a new one. Mutually exclusive with `read_only`. Defaults to False.
 
         Examples::
             >>> # create a hdf file to write
@@ -514,7 +486,7 @@ class HDF_File(HDF_Group):
             >>> hdf_file.dfs.df1 = pd.DataFrame({'a':[1,2,3]})
             >>> # write another DataFrame dataset into the dfs
             >>> hdf_file.dfs.df2 = pd.DataFrame({'a':[3,2,1]})
-            >>> # set an property value to the dataframe
+            >>> # set a property value to the dataframe
             >>> hdf_file.dfs.df1.data_from = "colleagues"
             >>> # get a dataframe dataset from a dfs
             >>> df1 = hdf_file.dfs.df1.values
@@ -526,12 +498,21 @@ class HDF_File(HDF_Group):
             >>> hdf_file.dfs.df1.data_from
             "colleagues"
         """
-        if delete_existing:
+        if read_only and (delete_existing or truncate):
+            raise ValueError(
+                "Parameters 'delete_existing'/'truncate' are mutually exclusive with 'read_only'."
+            )
+
+        if read_only:
+            mode = "r"
+        elif delete_existing:
             mode = "w"
         else:
             mode = "a"
-        with h5py.File(file_name, mode):#, swmr=True):
+
+        with h5py.File(file_name, mode):
             pass
+
         super().__init__(
             file_name=file_name,
             name="/",
